@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2012 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,7 +13,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import with_statement
 import functools
 import errno
 import os
@@ -22,6 +21,7 @@ import signal
 import time
 import subprocess
 import re
+from swift import gettext_ as _
 
 from swift.common.utils import search_tree, remove_file, write_file
 
@@ -30,10 +30,11 @@ RUN_DIR = '/var/run/swift'
 
 # auth-server has been removed from ALL_SERVERS, start it explicitly
 ALL_SERVERS = ['account-auditor', 'account-server', 'container-auditor',
-    'container-replicator', 'container-server', 'container-sync',
-    'container-updater', 'object-auditor', 'object-server', 'object-expirer',
-    'object-replicator', 'object-updater', 'proxy-server',
-    'account-replicator', 'account-reaper']
+               'container-replicator', 'container-reconciler',
+               'container-server', 'container-sync',
+               'container-updater', 'object-auditor', 'object-server',
+               'object-expirer', 'object-replicator', 'object-updater',
+               'proxy-server', 'account-replicator', 'account-reaper']
 MAIN_SERVERS = ['proxy-server', 'account-server', 'container-server',
                 'object-server']
 REST_SERVERS = [s for s in ALL_SERVERS if s not in MAIN_SERVERS]
@@ -41,13 +42,14 @@ GRACEFUL_SHUTDOWN_SERVERS = MAIN_SERVERS + ['auth-server']
 START_ONCE_SERVERS = REST_SERVERS
 # These are servers that match a type (account-*, container-*, object-*) but
 # don't use that type-server.conf file and instead use their own.
-STANDALONE_SERVERS = ['object-expirer']
+STANDALONE_SERVERS = ['object-expirer', 'container-reconciler']
 
-KILL_WAIT = 15  # seconds to wait for servers to die
+KILL_WAIT = 15  # seconds to wait for servers to die (by default)
 WARNING_WAIT = 3  # seconds to wait after message that may just be a warning
 
 MAX_DESCRIPTORS = 32768
 MAX_MEMORY = (1024 * 1024 * 1024) * 2  # 2 GB
+MAX_PROCS = 8192  # workers * disks * threads_per_disk, can get high
 
 
 def setup_env():
@@ -55,14 +57,27 @@ def setup_env():
     """
     try:
         resource.setrlimit(resource.RLIMIT_NOFILE,
-                (MAX_DESCRIPTORS, MAX_DESCRIPTORS))
-        resource.setrlimit(resource.RLIMIT_DATA,
-                (MAX_MEMORY, MAX_MEMORY))
+                           (MAX_DESCRIPTORS, MAX_DESCRIPTORS))
     except ValueError:
-        print _("WARNING: Unable to increase file descriptor limit.  "
+        print _("WARNING: Unable to modify file descriptor limit.  "
                 "Running as non-root?")
 
-    os.environ['PYTHON_EGG_CACHE'] = '/tmp'
+    try:
+        resource.setrlimit(resource.RLIMIT_DATA,
+                           (MAX_MEMORY, MAX_MEMORY))
+    except ValueError:
+        print _("WARNING: Unable to modify memory limit.  "
+                "Running as non-root?")
+
+    try:
+        resource.setrlimit(resource.RLIMIT_NPROC,
+                           (MAX_PROCS, MAX_PROCS))
+    except ValueError:
+        print _("WARNING: Unable to modify max process limit.  "
+                "Running as non-root?")
+
+    # Set PYTHON_EGG_CACHE if it isn't already set
+    os.environ.setdefault('PYTHON_EGG_CACHE', '/tmp')
 
 
 def command(func):
@@ -82,7 +97,7 @@ def command(func):
 
 
 def watch_server_pids(server_pids, interval=1, **kwargs):
-    """Monitor a collection of server pids yeilding back those pids that
+    """Monitor a collection of server pids yielding back those pids that
     aren't responding to signals.
 
     :param server_pids: a dict, lists of pids [int,...] keyed on
@@ -98,7 +113,7 @@ def watch_server_pids(server_pids, interval=1, **kwargs):
                 try:
                     # let pid stop if it wants to
                     os.waitpid(pid, os.WNOHANG)
-                except OSError, e:
+                except OSError as e:
                     if e.errno not in (errno.ECHILD, errno.ESRCH):
                         raise  # else no such child/process
             # check running pids for server
@@ -122,32 +137,36 @@ class UnknownCommandError(Exception):
     pass
 
 
-class Manager():
+class Manager(object):
     """Main class for performing commands on groups of servers.
 
     :param servers: list of server names as strings
 
     """
 
-    def __init__(self, servers):
-        server_names = set()
+    def __init__(self, servers, run_dir=RUN_DIR):
+        self.server_names = set()
         for server in servers:
             if server == 'all':
-                server_names.update(ALL_SERVERS)
+                self.server_names.update(ALL_SERVERS)
             elif server == 'main':
-                server_names.update(MAIN_SERVERS)
+                self.server_names.update(MAIN_SERVERS)
             elif server == 'rest':
-                server_names.update(REST_SERVERS)
+                self.server_names.update(REST_SERVERS)
             elif '*' in server:
                 # convert glob to regex
-                server_names.update([s for s in ALL_SERVERS if
-                                     re.match(server.replace('*', '.*'), s)])
+                self.server_names.update([
+                    s for s in ALL_SERVERS if
+                    re.match(server.replace('*', '.*'), s)])
             else:
-                server_names.add(server)
+                self.server_names.add(server)
 
         self.servers = set()
-        for name in server_names:
-            self.servers.add(Server(name))
+        for name in self.server_names:
+            self.servers.add(Server(name, run_dir))
+
+    def __iter__(self):
+        return iter(self.servers)
 
     @command
     def status(self, **kwargs):
@@ -218,12 +237,14 @@ class Manager():
                          for p in pids]
         # keep track of the pids yeiled back as killed for all servers
         killed_pids = set()
+        kill_wait = kwargs.get('kill_wait', KILL_WAIT)
         for server, killed_pid in watch_server_pids(server_pids,
-                                                interval=KILL_WAIT, **kwargs):
+                                                    interval=kill_wait,
+                                                    **kwargs):
             print _("%s (%s) appears to have stopped") % (server, killed_pid)
             killed_pids.add(killed_pid)
             if not killed_pids.symmetric_difference(signaled_pids):
-                # all proccesses have been stopped
+                # all processes have been stopped
                 return 0
 
         # reached interval n watch_pids w/o killing all servers
@@ -231,9 +252,20 @@ class Manager():
             if not killed_pids.issuperset(pids):
                 # some pids of this server were not killed
                 print _('Waited %s seconds for %s to die; giving up') % (
-                    KILL_WAIT, server)
+                    kill_wait, server)
         return 1
 
+    @command
+    def kill(self, **kwargs):
+        """stop a server (no error if not running)
+        """
+        status = self.stop(**kwargs)
+        kwargs['quiet'] = True
+        if status and not self.status(**kwargs):
+            # only exit error if the server is still running
+            return status
+        return 0
+
     @command
     def shutdown(self, **kwargs):
         """allow current requests to finish on supporting servers
@@ -258,8 +290,8 @@ class Manager():
         """
         kwargs['graceful'] = True
         status = 0
-        for server in self.servers:
-            m = Manager([server.server])
+        for server in self.server_names:
+            m = Manager([server])
             status += m.stop(**kwargs)
             status += m.start(**kwargs)
         return status
@@ -308,19 +340,24 @@ class Manager():
         return f(**kwargs)
 
 
-class Server():
+class Server(object):
     """Manage operations on a server or group of servers of similar type
 
     :param server: name of server
     """
 
-    def __init__(self, server):
-        if '-' not in server:
-            server = '%s-server' % server
+    def __init__(self, server, run_dir=RUN_DIR):
         self.server = server.lower()
-        self.type = server.rsplit('-', 1)[0]
-        self.cmd = 'swift-%s' % server
+        if '.' in self.server:
+            self.server, self.conf = self.server.rsplit('.', 1)
+        else:
+            self.conf = None
+        if '-' not in self.server:
+            self.server = '%s-server' % self.server
+        self.type = self.server.rsplit('-', 1)[0]
+        self.cmd = 'swift-%s' % self.server
         self.procs = []
+        self.run_dir = run_dir
 
     def __str__(self):
         return self.server
@@ -346,9 +383,9 @@ class Server():
 
         """
         return conf_file.replace(
-            os.path.normpath(SWIFT_DIR), RUN_DIR, 1).replace(
-                '%s-server' % self.type, self.server, 1).rsplit(
-                    '.conf', 1)[0] + '.pid'
+            os.path.normpath(SWIFT_DIR), self.run_dir, 1).replace(
+                '%s-server' % self.type, self.server, 1).replace(
+                    '.conf', '.pid', 1)
 
     def get_conf_file_name(self, pid_file):
         """Translate pid_file to a corresponding conf_file
@@ -360,13 +397,13 @@ class Server():
         """
         if self.server in STANDALONE_SERVERS:
             return pid_file.replace(
-                os.path.normpath(RUN_DIR), SWIFT_DIR, 1).rsplit(
-                        '.pid', 1)[0] + '.conf'
+                os.path.normpath(self.run_dir), SWIFT_DIR, 1).replace(
+                    '.pid', '.conf', 1)
         else:
             return pid_file.replace(
-                os.path.normpath(RUN_DIR), SWIFT_DIR, 1).replace(
-                    self.server, '%s-server' % self.type, 1).rsplit(
-                        '.pid', 1)[0] + '.conf'
+                os.path.normpath(self.run_dir), SWIFT_DIR, 1).replace(
+                    self.server, '%s-server' % self.type, 1).replace(
+                        '.pid', '.conf', 1)
 
     def conf_files(self, **kwargs):
         """Get conf files for this server
@@ -376,11 +413,16 @@ class Server():
         :returns: list of conf files
         """
         if self.server in STANDALONE_SERVERS:
-            found_conf_files = search_tree(SWIFT_DIR, self.server + '*',
-                                           '.conf')
+            server_search = self.server
         else:
-            found_conf_files = search_tree(SWIFT_DIR, '%s-server*' % self.type,
-                                          '.conf')
+            server_search = "%s-server" % self.type
+        if self.conf is not None:
+            found_conf_files = search_tree(SWIFT_DIR, server_search,
+                                           self.conf + '.conf',
+                                           dir_ext=self.conf + '.conf.d')
+        else:
+            found_conf_files = search_tree(SWIFT_DIR, server_search + '*',
+                                           '.conf', dir_ext='.conf.d')
         number = kwargs.get('number')
         if number:
             try:
@@ -409,7 +451,12 @@ class Server():
 
         :returns: list of pid files
         """
-        pid_files = search_tree(RUN_DIR, '%s*' % self.server, '.pid')
+        if self.conf is not None:
+            pid_files = search_tree(self.run_dir, '%s*' % self.server,
+                                    exts=[self.conf + '.pid',
+                                          self.conf + '.pid.d'])
+        else:
+            pid_files = search_tree(self.run_dir, '%s*' % self.server)
         if kwargs.get('number', 0):
             conf_files = self.conf_files(**kwargs)
             # filter pid_files to match the index of numbered conf_file
@@ -438,12 +485,14 @@ class Server():
                     print _('Signal %s  pid: %s  signal: %s') % (self.server,
                                                                  pid, sig)
                 os.kill(pid, sig)
-            except OSError, e:
+            except OSError as e:
                 if e.errno == errno.ESRCH:
                     # pid does not exist
                     if kwargs.get('verbose'):
                         print _("Removing stale pid file %s") % pid_file
                     remove_file(pid_file)
+                elif e.errno == errno.EPERM:
+                    print _("No permission to signal PID %d") % pid
             else:
                 # process exists
                 pids[pid] = pid_file
@@ -504,7 +553,7 @@ class Server():
         :param conf_file: path to conf_file to use as first arg
         :param once: boolean, add once argument to command
         :param wait: boolean, if true capture stdout with a pipe
-        :param daemon: boolean, if true ask server to log to console
+        :param daemon: boolean, if false ask server to log to console
 
         :returns : the pid of the spawned process
         """
@@ -539,8 +588,13 @@ class Server():
         """
         status = 0
         for proc in self.procs:
-            # wait for process to close it's stdout
+            # wait for process to close its stdout
             output = proc.stdout.read()
+            if kwargs.get('once', False):
+                # if you don't want once to wait you can send it to the
+                # background on the command line, I generally just run with
+                # no-daemon anyway, but this is quieter
+                proc.wait()
             if output:
                 print output
                 start = time.time()
@@ -570,7 +624,7 @@ class Server():
         """
         conf_files = self.conf_files(**kwargs)
         if not conf_files:
-            return []
+            return {}
 
         pids = self.get_running_pids(**kwargs)
 
@@ -590,7 +644,7 @@ class Server():
 
         if already_started:
             print _("%s already started...") % self.server
-            return []
+            return {}
 
         if self.server not in START_ONCE_SERVERS:
             kwargs['once'] = False
@@ -604,11 +658,13 @@ class Server():
             print '%s...(%s)' % (msg, conf_file)
             try:
                 pid = self.spawn(conf_file, **kwargs)
-            except OSError, e:
+            except OSError as e:
                 if e.errno == errno.ENOENT:
-                    # TODO: should I check if self.cmd exists earlier?
+                    #TODO(clayg): should I check if self.cmd exists earlier?
                     print _("%s does not exist") % self.cmd
                     break
+                else:
+                    raise
             pids[pid] = conf_file
 
         return pids
