@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2012 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,57 +14,28 @@
 # limitations under the License.
 
 import os
+import uuid
+from swift import gettext_ as _
 from time import ctime, time
-from random import random, shuffle
+from random import choice, random, shuffle
 from struct import unpack_from
 
 from eventlet import sleep, Timeout
 
-from swift.container import server as container_server
-from swift.common.client import ClientException, delete_object, put_object, \
-    quote
+import swift.common.db
+from swift.container.backend import ContainerBroker, DATADIR
+from swift.common.container_sync_realms import ContainerSyncRealms
 from swift.common.direct_client import direct_get_object
+from swift.common.internal_client import delete_object, put_object
+from swift.common.exceptions import ClientException
 from swift.common.ring import Ring
-from swift.common.db import ContainerBroker
-from swift.common.utils import audit_location_generator, get_logger, \
-    hash_path, normalize_timestamp, TRUE_VALUES, validate_sync_to, whataremyips
+from swift.common.utils import (
+    audit_location_generator, clean_content_type, config_true_value,
+    FileLikeIter, get_logger, hash_path, quote, urlparse, validate_sync_to,
+    whataremyips, Timestamp)
 from swift.common.daemon import Daemon
-
-
-class _Iter2FileLikeObject(object):
-    """
-    Returns an iterator's contents via :func:`read`, making it look like a file
-    object.
-    """
-
-    def __init__(self, iterator):
-        self.iterator = iterator
-        self._chunk = ''
-
-    def read(self, size=-1):
-        """
-        read([size]) -> read at most size bytes, returned as a string.
-
-        If the size argument is negative or omitted, read until EOF is reached.
-        Notice that when in non-blocking mode, less data than what was
-        requested may be returned, even if no size parameter was given.
-        """
-        if size < 0:
-            chunk = self._chunk
-            self._chunk = ''
-            return chunk + ''.join(self.iterator)
-        chunk = self._chunk
-        self._chunk = ''
-        if chunk and len(chunk) <= size:
-            return chunk
-        try:
-            chunk += self.iterator.next()
-        except StopIteration:
-            pass
-        if len(chunk) <= size:
-            return chunk
-        self._chunk = chunk[size:]
-        return chunk[:size]
+from swift.common.http import HTTP_UNAUTHORIZED, HTTP_NOT_FOUND
+from swift.common.storage_policy import POLICIES
 
 
 class ContainerSync(Daemon):
@@ -129,11 +100,9 @@ class ContainerSync(Daemon):
                  section of the container-server.conf
     :param container_ring: If None, the <swift_dir>/container.ring.gz will be
                            loaded. This is overridden by unit tests.
-    :param object_ring: If None, the <swift_dir>/object.ring.gz will be loaded.
-                        This is overridden by unit tests.
     """
 
-    def __init__(self, conf, container_ring=None, object_ring=None):
+    def __init__(self, conf, container_ring=None):
         #: The dict of configuration values from the [container-sync] section
         #: of the container-server.conf.
         self.conf = conf
@@ -143,8 +112,7 @@ class ContainerSync(Daemon):
         self.devices = conf.get('devices', '/srv/node')
         #: Indicates whether mount points should be verified as actual mount
         #: points (normally true, false for tests and SAIO).
-        self.mount_check = \
-            conf.get('mount_check', 'true').lower() in TRUE_VALUES
+        self.mount_check = config_true_value(conf.get('mount_check', 'true'))
         #: Minimum time between full scans. This is to keep the daemon from
         #: running wild on near empty systems.
         self.interval = int(conf.get('interval', 300))
@@ -152,11 +120,22 @@ class ContainerSync(Daemon):
         #: to the next one. If a conatiner sync hasn't finished in this time,
         #: it'll just be resumed next scan.
         self.container_time = int(conf.get('container_time', 60))
-        #: The list of hosts we're allowed to send syncs to.
-        self.allowed_sync_hosts = [h.strip()
+        #: ContainerSyncCluster instance for validating sync-to values.
+        self.realms_conf = ContainerSyncRealms(
+            os.path.join(
+                conf.get('swift_dir', '/etc/swift'),
+                'container-sync-realms.conf'),
+            self.logger)
+        #: The list of hosts we're allowed to send syncs to. This can be
+        #: overridden by data in self.realms_conf
+        self.allowed_sync_hosts = [
+            h.strip()
             for h in conf.get('allowed_sync_hosts', '127.0.0.1').split(',')
             if h.strip()]
-        self.proxy = conf.get('sync_proxy')
+        self.http_proxies = [
+            a.strip()
+            for a in conf.get('sync_proxy', '').split(',')
+            if a.strip()]
         #: Number of containers with sync turned on that were successfully
         #: synced.
         self.container_syncs = 0
@@ -170,25 +149,32 @@ class ContainerSync(Daemon):
         self.container_failures = 0
         #: Time of last stats report.
         self.reported = time()
-        swift_dir = conf.get('swift_dir', '/etc/swift')
+        self.swift_dir = conf.get('swift_dir', '/etc/swift')
         #: swift.common.ring.Ring for locating containers.
-        self.container_ring = container_ring or \
-            Ring(os.path.join(swift_dir, 'container.ring.gz'))
-        #: swift.common.ring.Ring for locating objects.
-        self.object_ring = object_ring or \
-            Ring(os.path.join(swift_dir, 'object.ring.gz'))
+        self.container_ring = container_ring or Ring(self.swift_dir,
+                                                     ring_name='container')
         self._myips = whataremyips()
         self._myport = int(conf.get('bind_port', 6001))
+        swift.common.db.DB_PREALLOCATION = \
+            config_true_value(conf.get('db_preallocation', 'f'))
 
-    def run_forever(self):
+    def get_object_ring(self, policy_idx):
+        """
+        Get the ring object to use based on its policy.
+
+        :policy_idx: policy index as defined in swift.conf
+        :returns: appropriate ring object
+        """
+        return POLICIES.get_object_ring(policy_idx, self.swift_dir)
+
+    def run_forever(self, *args, **kwargs):
         """
         Runs container sync scans until stopped.
         """
         sleep(random() * self.interval)
         while True:
             begin = time()
-            all_locs = audit_location_generator(self.devices,
-                                                container_server.DATADIR,
+            all_locs = audit_location_generator(self.devices, DATADIR, '.db',
                                                 mount_check=self.mount_check,
                                                 logger=self.logger)
             for path, device, partition in all_locs:
@@ -199,14 +185,13 @@ class ContainerSync(Daemon):
             if elapsed < self.interval:
                 sleep(self.interval - elapsed)
 
-    def run_once(self):
+    def run_once(self, *args, **kwargs):
         """
         Runs a single container sync scan.
         """
         self.logger.info(_('Begin container sync "once" mode'))
         begin = time()
-        all_locs = audit_location_generator(self.devices,
-                                            container_server.DATADIR,
+        all_locs = audit_location_generator(self.devices, DATADIR, '.db',
                                             mount_check=self.mount_check,
                                             logger=self.logger)
         for path, device, partition in all_locs:
@@ -247,9 +232,8 @@ class ContainerSync(Daemon):
 
         :param path: the path to a container db
         """
+        broker = None
         try:
-            if not path.endswith('.db'):
-                return
             broker = ContainerBroker(path)
             info = broker.get_info()
             x, nodes = self.container_ring.get_nodes(info['account'],
@@ -261,47 +245,54 @@ class ContainerSync(Daemon):
                 return
             if not broker.is_deleted():
                 sync_to = None
-                sync_key = None
+                user_key = None
                 sync_point1 = info['x_container_sync_point1']
                 sync_point2 = info['x_container_sync_point2']
                 for key, (value, timestamp) in broker.metadata.iteritems():
                     if key.lower() == 'x-container-sync-to':
                         sync_to = value
                     elif key.lower() == 'x-container-sync-key':
-                        sync_key = value
-                if not sync_to or not sync_key:
+                        user_key = value
+                if not sync_to or not user_key:
                     self.container_skips += 1
+                    self.logger.increment('skips')
                     return
-                sync_to = sync_to.rstrip('/')
-                err = validate_sync_to(sync_to, self.allowed_sync_hosts)
+                err, sync_to, realm, realm_key = validate_sync_to(
+                    sync_to, self.allowed_sync_hosts, self.realms_conf)
                 if err:
                     self.logger.info(
                         _('ERROR %(db_file)s: %(validate_sync_to_err)s'),
-                        {'db_file': broker.db_file,
+                        {'db_file': str(broker),
                          'validate_sync_to_err': err})
                     self.container_failures += 1
+                    self.logger.increment('failures')
                     return
                 stop_at = time() + self.container_time
+                next_sync_point = None
                 while time() < stop_at and sync_point2 < sync_point1:
                     rows = broker.get_items_since(sync_point2, 1)
                     if not rows:
                         break
                     row = rows[0]
-                    if row['ROWID'] >= sync_point1:
+                    if row['ROWID'] > sync_point1:
                         break
                     key = hash_path(info['account'], info['container'],
                                     row['name'], raw_digest=True)
-                    # This node will only intially sync out one third of the
-                    # objects (if 3 replicas, 1/4 if 4, etc.). This section
-                    # will attempt to sync previously skipped rows in case the
-                    # other nodes didn't succeed.
-                    if unpack_from('>I', key)[0] % \
-                            self.container_ring.replica_count != ordinal:
-                        if not self.container_sync_row(row, sync_to, sync_key,
-                                                       broker, info):
-                            return
+                    # This node will only initially sync out one third of the
+                    # objects (if 3 replicas, 1/4 if 4, etc.) and will skip
+                    # problematic rows as needed in case of faults.
+                    # This section will attempt to sync previously skipped
+                    # rows in case the previous attempts by any of the nodes
+                    # didn't succeed.
+                    if not self.container_sync_row(
+                            row, sync_to, user_key, broker, info, realm,
+                            realm_key):
+                        if not next_sync_point:
+                            next_sync_point = sync_point2
                     sync_point2 = row['ROWID']
                     broker.set_x_container_sync_points(None, sync_point2)
+                if next_sync_point:
+                    broker.set_x_container_sync_points(None, next_sync_point)
                 while time() < stop_at:
                     rows = broker.get_items_since(sync_point1, 1)
                     if not rows:
@@ -309,79 +300,113 @@ class ContainerSync(Daemon):
                     row = rows[0]
                     key = hash_path(info['account'], info['container'],
                                     row['name'], raw_digest=True)
-                    # This node will only intially sync out one third of the
+                    # This node will only initially sync out one third of the
                     # objects (if 3 replicas, 1/4 if 4, etc.). It'll come back
                     # around to the section above and attempt to sync
                     # previously skipped rows in case the other nodes didn't
-                    # succeed.
+                    # succeed or in case it failed to do so the first time.
                     if unpack_from('>I', key)[0] % \
-                            self.container_ring.replica_count == ordinal:
-                        if not self.container_sync_row(row, sync_to, sync_key,
-                                                       broker, info):
-                            return
+                            len(nodes) == ordinal:
+                        self.container_sync_row(
+                            row, sync_to, user_key, broker, info, realm,
+                            realm_key)
                     sync_point1 = row['ROWID']
                     broker.set_x_container_sync_points(sync_point1, None)
                 self.container_syncs += 1
-        except (Exception, Timeout), err:
+                self.logger.increment('syncs')
+        except (Exception, Timeout) as err:
             self.container_failures += 1
-            self.logger.exception(_('ERROR Syncing %s'), (broker.db_file))
+            self.logger.increment('failures')
+            self.logger.exception(_('ERROR Syncing %s'),
+                                  broker if broker else path)
 
-    def container_sync_row(self, row, sync_to, sync_key, broker, info):
+    def container_sync_row(self, row, sync_to, user_key, broker, info,
+                           realm, realm_key):
         """
         Sends the update the row indicates to the sync_to container.
 
         :param row: The updated row in the local database triggering the sync
                     update.
         :param sync_to: The URL to the remote container.
-        :param sync_key: The X-Container-Sync-Key to use when sending requests
+        :param user_key: The X-Container-Sync-Key to use when sending requests
                          to the other container.
         :param broker: The local container database broker.
         :param info: The get_info result from the local container database
                      broker.
+        :param realm: The realm from self.realms_conf, if there is one.
+            If None, fallback to using the older allowed_sync_hosts
+            way of syncing.
+        :param realm_key: The realm key from self.realms_conf, if there
+            is one. If None, fallback to using the older
+            allowed_sync_hosts way of syncing.
         :returns: True on success
         """
         try:
+            start_time = time()
             if row['deleted']:
                 try:
-                    delete_object(sync_to, name=row['name'],
-                        headers={'x-timestamp': row['created_at'],
-                                 'x-container-sync-key': sync_key},
-                        proxy=self.proxy)
-                except ClientException, err:
-                    if err.http_status != 404:
+                    headers = {'x-timestamp': row['created_at']}
+                    if realm and realm_key:
+                        nonce = uuid.uuid4().hex
+                        path = urlparse(sync_to).path + '/' + quote(
+                            row['name'])
+                        sig = self.realms_conf.get_sig(
+                            'DELETE', path, headers['x-timestamp'], nonce,
+                            realm_key, user_key)
+                        headers['x-container-sync-auth'] = '%s %s %s' % (
+                            realm, nonce, sig)
+                    else:
+                        headers['x-container-sync-key'] = user_key
+                    delete_object(sync_to, name=row['name'], headers=headers,
+                                  proxy=self.select_http_proxy(),
+                                  logger=self.logger)
+                except ClientException as err:
+                    if err.http_status != HTTP_NOT_FOUND:
                         raise
                 self.container_deletes += 1
+                self.logger.increment('deletes')
+                self.logger.timing_since('deletes.timing', start_time)
             else:
-                part, nodes = self.object_ring.get_nodes(
-                    info['account'], info['container'],
-                    row['name'])
+                part, nodes = \
+                    self.get_object_ring(info['storage_policy_index']). \
+                    get_nodes(info['account'], info['container'],
+                              row['name'])
                 shuffle(nodes)
                 exc = None
-                looking_for_timestamp = float(row['created_at'])
+                looking_for_timestamp = Timestamp(row['created_at'])
                 timestamp = -1
                 headers = body = None
+                headers_out = {'X-Backend-Storage-Policy-Index':
+                               str(info['storage_policy_index'])}
                 for node in nodes:
                     try:
-                        these_headers, this_body = direct_get_object(node,
-                            part, info['account'], info['container'],
-                            row['name'], resp_chunk_size=65536)
-                        this_timestamp = float(these_headers['x-timestamp'])
+                        these_headers, this_body = direct_get_object(
+                            node, part, info['account'], info['container'],
+                            row['name'], headers=headers_out,
+                            resp_chunk_size=65536)
+                        this_timestamp = Timestamp(
+                            these_headers['x-timestamp'])
                         if this_timestamp > timestamp:
                             timestamp = this_timestamp
                             headers = these_headers
                             body = this_body
-                    except ClientException, err:
+                    except ClientException as err:
                         # If any errors are not 404, make sure we report the
                         # non-404 one. We don't want to mistakenly assume the
                         # object no longer exists just because one says so and
                         # the others errored for some other reason.
-                        if not exc or exc.http_status == 404:
+                        if not exc or getattr(
+                                exc, 'http_status', HTTP_NOT_FOUND) == \
+                                HTTP_NOT_FOUND:
                             exc = err
+                    except (Exception, Timeout) as err:
+                        exc = err
                 if timestamp < looking_for_timestamp:
                     if exc:
                         raise exc
-                    raise Exception(_('Unknown exception trying to GET: '
-                        '%(node)r %(account)r %(container)r %(object)r'),
+                    raise Exception(
+                        _('Unknown exception trying to GET: %(node)r '
+                          '%(account)r %(container)r %(object)r'),
                         {'node': node, 'part': part,
                          'account': info['account'],
                          'container': info['container'],
@@ -391,34 +416,55 @@ class ContainerSync(Daemon):
                         del headers[key]
                 if 'etag' in headers:
                     headers['etag'] = headers['etag'].strip('"')
+                if 'content-type' in headers:
+                    headers['content-type'] = clean_content_type(
+                        headers['content-type'])
                 headers['x-timestamp'] = row['created_at']
-                headers['x-container-sync-key'] = sync_key
+                if realm and realm_key:
+                    nonce = uuid.uuid4().hex
+                    path = urlparse(sync_to).path + '/' + quote(row['name'])
+                    sig = self.realms_conf.get_sig(
+                        'PUT', path, headers['x-timestamp'], nonce, realm_key,
+                        user_key)
+                    headers['x-container-sync-auth'] = '%s %s %s' % (
+                        realm, nonce, sig)
+                else:
+                    headers['x-container-sync-key'] = user_key
                 put_object(sync_to, name=row['name'], headers=headers,
-                    contents=_Iter2FileLikeObject(body), proxy=self.proxy)
+                           contents=FileLikeIter(body),
+                           proxy=self.select_http_proxy(), logger=self.logger)
                 self.container_puts += 1
-        except ClientException, err:
-            if err.http_status == 401:
-                self.logger.info(_('Unauth %(sync_from)r '
-                    '=> %(sync_to)r'),
+                self.logger.increment('puts')
+                self.logger.timing_since('puts.timing', start_time)
+        except ClientException as err:
+            if err.http_status == HTTP_UNAUTHORIZED:
+                self.logger.info(
+                    _('Unauth %(sync_from)r => %(sync_to)r'),
                     {'sync_from': '%s/%s' %
                         (quote(info['account']), quote(info['container'])),
                      'sync_to': sync_to})
-            elif err.http_status == 404:
-                self.logger.info(_('Not found %(sync_from)r '
-                    '=> %(sync_to)r'),
+            elif err.http_status == HTTP_NOT_FOUND:
+                self.logger.info(
+                    _('Not found %(sync_from)r => %(sync_to)r \
+                      - object %(obj_name)r'),
                     {'sync_from': '%s/%s' %
                         (quote(info['account']), quote(info['container'])),
-                     'sync_to': sync_to})
+                     'sync_to': sync_to, 'obj_name': row['name']})
             else:
                 self.logger.exception(
                     _('ERROR Syncing %(db_file)s %(row)s'),
-                    {'db_file': broker.db_file, 'row': row})
+                    {'db_file': str(broker), 'row': row})
             self.container_failures += 1
+            self.logger.increment('failures')
             return False
-        except (Exception, Timeout), err:
+        except (Exception, Timeout) as err:
             self.logger.exception(
                 _('ERROR Syncing %(db_file)s %(row)s'),
-                {'db_file': broker.db_file, 'row': row})
+                {'db_file': str(broker), 'row': row})
             self.container_failures += 1
+            self.logger.increment('failures')
             return False
         return True
+
+    def select_http_proxy(self):
+        return choice(self.http_proxies) if self.http_proxies else None
