@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2012 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,7 +14,6 @@
 # limitations under the License.
 
 import unittest
-from nose import SkipTest
 from test.unit import temptree
 
 import os
@@ -22,7 +21,6 @@ import sys
 import resource
 import signal
 import errno
-from contextlib import contextmanager
 from collections import defaultdict
 from threading import Thread
 from time import sleep, time
@@ -32,7 +30,8 @@ from swift.common import manager
 DUMMY_SIG = 1
 
 
-class MockOs():
+class MockOs(object):
+    RAISE_EPERM_SIG = 99
 
     def __init__(self, pids):
         self.running_pids = pids
@@ -42,6 +41,8 @@ class MockOs():
         self.execlp_called = False
 
     def kill(self, pid, sig):
+        if sig == self.RAISE_EPERM_SIG:
+            raise OSError(errno.EPERM, 'Operation not permitted')
         if pid not in self.running_pids:
             raise OSError(3, 'No such process')
         self.pid_sigs[pid].append(sig)
@@ -76,7 +77,7 @@ class TestManagerModule(unittest.TestCase):
                           len(manager.REST_SERVERS))
 
     def test_setup_env(self):
-        class MockResource():
+        class MockResource(object):
             def __init__(self, error=None):
                 self.error = error
                 self.called_with_args = []
@@ -104,16 +105,20 @@ class TestManagerModule(unittest.TestCase):
                                           manager.MAX_DESCRIPTORS)),
                 (resource.RLIMIT_DATA, (manager.MAX_MEMORY,
                                         manager.MAX_MEMORY)),
+                (resource.RLIMIT_NPROC, (manager.MAX_PROCS,
+                                         manager.MAX_PROCS)),
             ]
             self.assertEquals(manager.resource.called_with_args, expected)
-            self.assertEquals(manager.os.environ['PYTHON_EGG_CACHE'], '/tmp')
+            self.assertTrue(
+                manager.os.environ['PYTHON_EGG_CACHE'].startswith('/tmp'))
 
             # test error condition
             manager.resource = MockResource(error=ValueError())
             manager.os.environ = {}
             manager.setup_env()
             self.assertEquals(manager.resource.called_with_args, [])
-            self.assertEquals(manager.os.environ['PYTHON_EGG_CACHE'], '/tmp')
+            self.assertTrue(
+                manager.os.environ['PYTHON_EGG_CACHE'].startswith('/tmp'))
 
             manager.resource = MockResource(error=OSError())
             manager.os.environ = {}
@@ -139,10 +144,12 @@ class TestManagerModule(unittest.TestCase):
         self.assert_(myfunc.publicly_accessible)
 
     def test_watch_server_pids(self):
-        class MockOs():
+        class MockOs(object):
             WNOHANG = os.WNOHANG
 
-            def __init__(self, pid_map={}):
+            def __init__(self, pid_map=None):
+                if pid_map is None:
+                    pid_map = {}
                 self.pid_map = {}
                 for pid, v in pid_map.items():
                     self.pid_map[pid] = (x for x in v)
@@ -159,7 +166,7 @@ class TestManagerModule(unittest.TestCase):
                 else:
                     return rv
 
-        class MockTime():
+        class MockTime(object):
             def __init__(self, ticks=None):
                 self.tock = time()
                 if not ticks:
@@ -177,9 +184,9 @@ class TestManagerModule(unittest.TestCase):
             def sleep(*args):
                 return
 
-        class MockServer():
+        class MockServer(object):
 
-            def __init__(self, pids, zombie=0):
+            def __init__(self, pids, run_dir=manager.RUN_DIR, zombie=0):
                 self.heartbeat = (pids for _ in range(zombie))
 
             def get_running_pids(self):
@@ -309,6 +316,24 @@ class TestServer(unittest.TestCase):
             'container-auditor/1/container-auditor.pid')
         self.assertEquals(pid_file, server.get_pid_file_name(conf_file))
 
+    def test_get_custom_pid_file_name(self):
+        random_run_dir = "/random/dir"
+        get_random_run_dir = lambda x: os.path.join(random_run_dir, x)
+        server = manager.Server('proxy', run_dir=random_run_dir)
+        conf_file = self.join_swift_dir('proxy-server.conf')
+        pid_file = get_random_run_dir('proxy-server.pid')
+        self.assertEquals(pid_file, server.get_pid_file_name(conf_file))
+        server = manager.Server('object-replicator', run_dir=random_run_dir)
+        conf_file = self.join_swift_dir('object-server/1.conf')
+        pid_file = get_random_run_dir('object-replicator/1.pid')
+        self.assertEquals(pid_file, server.get_pid_file_name(conf_file))
+        server = manager.Server('container-auditor', run_dir=random_run_dir)
+        conf_file = self.join_swift_dir(
+            'container-server/1/container-auditor.conf')
+        pid_file = get_random_run_dir(
+            'container-auditor/1/container-auditor.pid')
+        self.assertEquals(pid_file, server.get_pid_file_name(conf_file))
+
     def test_get_conf_file_name(self):
         server = manager.Server('proxy')
         conf_file = self.join_swift_dir('proxy-server.conf')
@@ -389,6 +414,22 @@ class TestServer(unittest.TestCase):
             conf_files = server.conf_files(number=5)
             self.assertFalse(conf_files)
 
+        # test getting specific conf
+        conf_files = (
+            'account-server/1.conf',
+            'account-server/2.conf',
+            'account-server/3.conf',
+            'account-server/4.conf',
+        )
+        with temptree(conf_files) as t:
+            manager.SWIFT_DIR = t
+            server = manager.Server('account.2')
+            conf_files = server.conf_files()
+            self.assertEquals(len(conf_files), 1)
+            conf_file = conf_files[0]
+            self.assertEquals(conf_file,
+                              self.join_swift_dir('account-server/2.conf'))
+
         # test verbose & quiet
         conf_files = (
             'auth-server.ini',
@@ -434,6 +475,96 @@ class TestServer(unittest.TestCase):
             conf = self.join_swift_dir(server_name + '.conf')
             self.assertEquals(conf_file, conf)
 
+    def test_proxy_conf_dir(self):
+        conf_files = (
+            'proxy-server.conf.d/00.conf',
+            'proxy-server.conf.d/01.conf',
+        )
+        with temptree(conf_files) as t:
+            manager.SWIFT_DIR = t
+            server = manager.Server('proxy')
+            conf_dirs = server.conf_files()
+            self.assertEquals(len(conf_dirs), 1)
+            conf_dir = conf_dirs[0]
+            proxy_conf_dir = self.join_swift_dir('proxy-server.conf.d')
+            self.assertEquals(proxy_conf_dir, conf_dir)
+
+    def test_named_conf_dir(self):
+        conf_files = (
+            'object-server/base.conf-template',
+            'object-server/object-server.conf.d/00_base.conf',
+            'object-server/object-server.conf.d/10_server.conf',
+            'object-server/object-replication.conf.d/00_base.conf',
+            'object-server/object-replication.conf.d/10_server.conf',
+        )
+        with temptree(conf_files) as t:
+            manager.SWIFT_DIR = t
+            server = manager.Server('object.replication')
+            conf_dirs = server.conf_files()
+            self.assertEquals(len(conf_dirs), 1)
+            conf_dir = conf_dirs[0]
+            replication_server_conf_dir = self.join_swift_dir(
+                'object-server/object-replication.conf.d')
+            self.assertEquals(replication_server_conf_dir, conf_dir)
+            # and again with no named filter
+            server = manager.Server('object')
+            conf_dirs = server.conf_files()
+            self.assertEquals(len(conf_dirs), 2)
+            for named_conf in ('server', 'replication'):
+                conf_dir = self.join_swift_dir(
+                    'object-server/object-%s.conf.d' % named_conf)
+                self.assert_(conf_dir in conf_dirs)
+
+    def test_conf_dir(self):
+        conf_files = (
+            'object-server/object-server.conf-base',
+            'object-server/1.conf.d/base.conf',
+            'object-server/1.conf.d/1.conf',
+            'object-server/2.conf.d/base.conf',
+            'object-server/2.conf.d/2.conf',
+            'object-server/3.conf.d/base.conf',
+            'object-server/3.conf.d/3.conf',
+            'object-server/4.conf.d/base.conf',
+            'object-server/4.conf.d/4.conf',
+        )
+        with temptree(conf_files) as t:
+            manager.SWIFT_DIR = t
+            server = manager.Server('object-replicator')
+            conf_dirs = server.conf_files()
+            self.assertEquals(len(conf_dirs), 4)
+            c1 = self.join_swift_dir('object-server/1.conf.d')
+            c2 = self.join_swift_dir('object-server/2.conf.d')
+            c3 = self.join_swift_dir('object-server/3.conf.d')
+            c4 = self.join_swift_dir('object-server/4.conf.d')
+            for c in [c1, c2, c3, c4]:
+                self.assert_(c in conf_dirs)
+            # test configs returned sorted
+            sorted_confs = sorted([c1, c2, c3, c4])
+            self.assertEquals(conf_dirs, sorted_confs)
+
+    def test_named_conf_dir_pid_files(self):
+        conf_files = (
+            'object-server/object-server.pid.d',
+            'object-server/object-replication.pid.d',
+        )
+        with temptree(conf_files) as t:
+            manager.RUN_DIR = t
+            server = manager.Server('object.replication', run_dir=t)
+            pid_files = server.pid_files()
+            self.assertEquals(len(pid_files), 1)
+            pid_file = pid_files[0]
+            replication_server_pid = self.join_run_dir(
+                'object-server/object-replication.pid.d')
+            self.assertEquals(replication_server_pid, pid_file)
+            # and again with no named filter
+            server = manager.Server('object', run_dir=t)
+            pid_files = server.pid_files()
+            self.assertEquals(len(pid_files), 2)
+            for named_pid in ('server', 'replication'):
+                pid_file = self.join_run_dir(
+                    'object-server/object-%s.pid.d' % named_pid)
+                self.assert_(pid_file in pid_files)
+
     def test_iter_pid_files(self):
         """
         Server.iter_pid_files is kinda boring, test the
@@ -448,7 +579,7 @@ class TestServer(unittest.TestCase):
         files, contents = zip(*pid_files)
         with temptree(files, contents) as t:
             manager.RUN_DIR = t
-            server = manager.Server('proxy')
+            server = manager.Server('proxy', run_dir=t)
             # test get one file
             iter = server.iter_pid_files()
             pid_file, pid = iter.next()
@@ -457,13 +588,13 @@ class TestServer(unittest.TestCase):
             # ... and only one file
             self.assertRaises(StopIteration, iter.next)
             # test invalid value in pid file
-            server = manager.Server('auth')
+            server = manager.Server('auth', run_dir=t)
             self.assertRaises(ValueError, server.iter_pid_files().next)
             # test object-server doesn't steal pids from object-replicator
-            server = manager.Server('object')
+            server = manager.Server('object', run_dir=t)
             self.assertRaises(StopIteration, server.iter_pid_files().next)
             # test multi-pid iter
-            server = manager.Server('object-replicator')
+            server = manager.Server('object-replicator', run_dir=t)
             real_map = {
                 11: self.join_run_dir('object-replicator/1.pid'),
                 12: self.join_run_dir('object-replicator/2.pid'),
@@ -492,7 +623,7 @@ class TestServer(unittest.TestCase):
             files, pids = zip(*pid_files)
             with temptree(files, pids) as t:
                 manager.RUN_DIR = t
-                server = manager.Server('object')
+                server = manager.Server('object', run_dir=t)
                 # test get all pid files
                 real_map = {
                     1: self.join_run_dir('object-server/1.pid'),
@@ -517,17 +648,46 @@ class TestServer(unittest.TestCase):
                 pids = list(server.iter_pid_files(number=5))
                 self.assertFalse(pids)
 
+        # test get pid_files by conf name
+        conf_files = (
+            'object-server/1.conf',
+            'object-server/2.conf',
+            'object-server/3.conf',
+            'object-server/4.conf',
+        )
+
+        pid_files = (
+            ('object-server/1.pid', 1),
+            ('object-server/2.pid', 2),
+            ('object-server/5.pid', 5),
+        )
+
+        with temptree(conf_files) as swift_dir:
+            manager.SWIFT_DIR = swift_dir
+            files, pids = zip(*pid_files)
+            with temptree(files, pids) as t:
+                manager.RUN_DIR = t
+                server = manager.Server('object.2', run_dir=t)
+                # test get pid with matching conf
+                pids = list(server.iter_pid_files())
+                self.assertEquals(len(pids), 1)
+                pid_file, pid = pids[0]
+                self.assertEquals(pid, 2)
+                pid_two = self.join_run_dir('object-server/2.pid')
+                self.assertEquals(pid_file, pid_two)
+
     def test_signal_pids(self):
         pid_files = (
             ('proxy-server.pid', 1),
             ('auth-server.pid', 2),
+            ('object-server.pid', 3),
         )
         files, pids = zip(*pid_files)
         with temptree(files, pids) as t:
             manager.RUN_DIR = t
             # mock os with both pids running
             manager.os = MockOs([1, 2])
-            server = manager.Server('proxy')
+            server = manager.Server('proxy', run_dir=t)
             pids = server.signal_pids(DUMMY_SIG)
             self.assertEquals(len(pids), 1)
             self.assert_(1 in pids)
@@ -559,13 +719,19 @@ class TestServer(unittest.TestCase):
                         self.join_run_dir('proxy-server.pid')))
                     # reset mock os with no running pids
                     manager.os = MockOs([])
-                    server = manager.Server('auth')
+                    server = manager.Server('auth', run_dir=t)
                     # test verbose warns on removing pid file
                     pids = server.signal_pids(signal.SIG_DFL, verbose=True)
                     output = pop_stream(f)
                     self.assert_('stale pid' in output.lower())
                     auth_pid = self.join_run_dir('auth-server.pid')
                     self.assert_(auth_pid in output)
+                    # test warning with insufficient permissions
+                    server = manager.Server('object', run_dir=t)
+                    pids = server.signal_pids(manager.os.RAISE_EPERM_SIG)
+                    output = pop_stream(f)
+                    self.assert_('no permission to signal pid 3' in
+                                 output.lower(), output)
             finally:
                 sys.stdout = old_stdout
 
@@ -577,14 +743,14 @@ class TestServer(unittest.TestCase):
         )
         with temptree(*zip(*pid_files)) as t:
             manager.RUN_DIR = t
-            server = manager.Server('test-server')
+            server = manager.Server('test-server', run_dir=t)
             # mock os, only pid '1' is running
             manager.os = MockOs([1])
             running_pids = server.get_running_pids()
             self.assertEquals(len(running_pids), 1)
             self.assert_(1 in running_pids)
             self.assert_(2 not in running_pids)
-            # test persistant running pid files
+            # test persistent running pid files
             self.assert_(os.path.exists(os.path.join(t, 'test-server1.pid')))
             # test clean up stale pids
             pid_two = self.join_swift_dir('test-server2.pid')
@@ -611,7 +777,7 @@ class TestServer(unittest.TestCase):
             manager.RUN_DIR = t
             # all pids are running
             manager.os = MockOs(pids)
-            server = manager.Server('thing-doer')
+            server = manager.Server('thing-doer', run_dir=t)
             running_pids = server.get_running_pids()
             # only thing-doer.pid, 1
             self.assertEquals(len(running_pids), 1)
@@ -627,7 +793,7 @@ class TestServer(unittest.TestCase):
                 self.assert_(os.path.exists, os.path.join(t, f))
             # verify that servers are in fact not running
             for server_name in ('thing-sayer', 'other-doer', 'other-sayer'):
-                server = manager.Server(server_name)
+                server = manager.Server(server_name, run_dir=t)
                 running_pids = server.get_running_pids()
                 self.assertFalse(running_pids)
             # and now all OTHER pid files are cleaned out
@@ -644,7 +810,7 @@ class TestServer(unittest.TestCase):
         files, running_pids = zip(*pid_files)
         with temptree(files, running_pids) as t:
             manager.RUN_DIR = t
-            server = manager.Server('object')
+            server = manager.Server('object', run_dir=t)
             # test no servers running
             manager.os = MockOs([])
             pids = server.kill_running_pids()
@@ -652,8 +818,10 @@ class TestServer(unittest.TestCase):
         files, running_pids = zip(*pid_files)
         with temptree(files, running_pids) as t:
             manager.RUN_DIR = t
+            server.run_dir = t
             # start up pid
             manager.os = MockOs([1])
+            server = manager.Server('object', run_dir=t)
             # test kill one pid
             pids = server.kill_running_pids()
             self.assertEquals(len(pids), 1)
@@ -673,7 +841,7 @@ class TestServer(unittest.TestCase):
             # test multi server kill & ignore graceful on unsupported server
             self.assertFalse('object-replicator' in
                              manager.GRACEFUL_SHUTDOWN_SERVERS)
-            server = manager.Server('object-replicator')
+            server = manager.Server('object-replicator', run_dir=t)
             pids = server.kill_running_pids(graceful=True)
             self.assertEquals(len(pids), 2)
             for pid in (11, 12):
@@ -704,7 +872,7 @@ class TestServer(unittest.TestCase):
             with temptree(files, pids) as t:
                 manager.RUN_DIR = t
                 # setup running servers
-                server = manager.Server('test')
+                server = manager.Server('test', run_dir=t)
                 # capture stdio
                 old_stdout = sys.stdout
                 try:
@@ -770,7 +938,7 @@ class TestServer(unittest.TestCase):
     def test_spawn(self):
 
         # mocks
-        class MockProcess():
+        class MockProcess(object):
 
             NOTHING = 'default besides None'
             STDOUT = 'stdout'
@@ -784,7 +952,7 @@ class TestServer(unittest.TestCase):
             def Popen(self, args, **kwargs):
                 return MockProc(self.pids.next(), args, **kwargs)
 
-        class MockProc():
+        class MockProc(object):
 
             def __init__(self, pid, args, stdout=MockProcess.NOTHING,
                          stderr=MockProcess.NOTHING):
@@ -803,6 +971,7 @@ class TestServer(unittest.TestCase):
             manager.SWIFT_DIR = swift_dir
             with temptree([]) as t:
                 manager.RUN_DIR = t
+                server.run_dir = t
                 old_subprocess = manager.subprocess
                 try:
                     # test single server process calls spawn once
@@ -833,7 +1002,7 @@ class TestServer(unittest.TestCase):
                     conf2 = self.join_swift_dir('test-server/2.conf')
                     conf3 = self.join_swift_dir('test-server/3.conf')
                     conf4 = self.join_swift_dir('test-server/4.conf')
-                    server = manager.Server('test')
+                    server = manager.Server('test', run_dir=t)
                     # test server run once
                     server.spawn(conf1, once=True)
                     self.assert_(server.procs)
@@ -940,7 +1109,7 @@ class TestServer(unittest.TestCase):
                 print >>self._stdout, 'mock process finished'
                 self.finished = True
 
-        class MockTime():
+        class MockTime(object):
 
             def time(self):
                 return time()
@@ -956,7 +1125,7 @@ class TestServer(unittest.TestCase):
                 manager.WARNING_WAIT = 0.01
                 manager.time = MockTime()
                 with open(os.path.join(t, 'output'), 'w+') as f:
-                    # acctually capture the read stdout (for prints)
+                    # actually capture the read stdout (for prints)
                     sys.stdout = f
                     # test closing pipe in subprocess unblocks read
                     with MockProcess() as proc:
@@ -983,7 +1152,7 @@ class TestServer(unittest.TestCase):
                         self.assertEquals(status, 1)
                     self.assert_('failed' in pop_stream(f))
                     # test multiple procs
-                    procs = [MockProcess() for i in range(3)]
+                    procs = [MockProcess(delay=.5) for i in range(3)]
                     for proc in procs:
                         proc.start()
                     server.procs = procs
@@ -999,7 +1168,7 @@ class TestServer(unittest.TestCase):
                 manager.time = old_time
 
     def test_interact(self):
-        class MockProcess():
+        class MockProcess(object):
 
             def __init__(self, fail=False):
                 self.returncode = None
@@ -1039,7 +1208,7 @@ class TestServer(unittest.TestCase):
         )
 
         #mocks
-        class MockSpawn():
+        class MockSpawn(object):
 
             def __init__(self, pids=None):
                 self.conf_files = []
@@ -1071,11 +1240,11 @@ class TestServer(unittest.TestCase):
                     with open(os.path.join(t, 'output'), 'w+') as f:
                         sys.stdout = f
                         # can't start server w/o an conf
-                        server = manager.Server('test')
+                        server = manager.Server('test', run_dir=t)
                         self.assertFalse(server.launch())
                         # start mock os running all pids
                         manager.os = MockOs(pids)
-                        server = manager.Server('proxy')
+                        server = manager.Server('proxy', run_dir=t)
                         # can't start server if it's already running
                         self.assertFalse(server.launch())
                         output = pop_stream(f)
@@ -1118,8 +1287,8 @@ class TestServer(unittest.TestCase):
                             4: conf4,
                         }
                         self.assertEquals(server.launch(once=True), expected)
-                        self.assertEquals(mock_spawn.conf_files, [conf1, conf2,
-                                                                 conf3, conf4])
+                        self.assertEquals(mock_spawn.conf_files, [
+                            conf1, conf2, conf3, conf4])
                         expected = {
                             'once': True,
                         }
@@ -1169,7 +1338,7 @@ class TestServer(unittest.TestCase):
                 manager.RUN_DIR = t
                 # start all pids in mock os
                 manager.os = MockOs(pids)
-                server = manager.Server('account-reaper')
+                server = manager.Server('account-reaper', run_dir=t)
                 # test kill all running pids
                 pids = server.stop()
                 self.assertEquals(len(pids), 4)
@@ -1240,7 +1409,7 @@ class TestManager(unittest.TestCase):
             self.assert_(server.server in manager.ALL_SERVERS)
         # test dedupe
         m = manager.Manager(['main', 'rest', 'proxy', 'object',
-                                           'container', 'account'])
+                             'container', 'account'])
         self.assertEquals(len(m.servers), len(manager.ALL_SERVERS))
         for server in m.servers:
             self.assert_(server.server in manager.ALL_SERVERS)
@@ -1257,10 +1426,16 @@ class TestManager(unittest.TestCase):
         for s in m.servers:
             self.assert_(str(s) in replicators)
 
-    def test_status(self):
-        class MockServer():
+    def test_iter(self):
+        m = manager.Manager(['all'])
+        self.assertEquals(len(list(m)), len(manager.ALL_SERVERS))
+        for server in m:
+            self.assert_(server.server in manager.ALL_SERVERS)
 
-            def __init__(self, server):
+    def test_status(self):
+        class MockServer(object):
+
+            def __init__(self, server, run_dir=manager.RUN_DIR):
                 self.server = server
                 self.called_kwargs = []
 
@@ -1294,13 +1469,14 @@ class TestManager(unittest.TestCase):
         def mock_setup_env():
             getattr(mock_setup_env, 'called', []).append(True)
 
-        class MockServer():
-            def __init__(self, server):
+        class MockServer(object):
+            def __init__(self, server, run_dir=manager.RUN_DIR):
                 self.server = server
                 self.called = defaultdict(list)
 
             def launch(self, **kwargs):
                 self.called['launch'].append(kwargs)
+                return {}
 
             def wait(self, **kwargs):
                 self.called['wait'].append(kwargs)
@@ -1356,13 +1532,14 @@ class TestManager(unittest.TestCase):
             manager.Server = old_swift_server
 
     def test_no_wait(self):
-        class MockServer():
-            def __init__(self, server):
+        class MockServer(object):
+            def __init__(self, server, run_dir=manager.RUN_DIR):
                 self.server = server
                 self.called = defaultdict(list)
 
             def launch(self, **kwargs):
                 self.called['launch'].append(kwargs)
+                return {}
 
             def wait(self, **kwargs):
                 self.called['wait'].append(kwargs)
@@ -1406,14 +1583,15 @@ class TestManager(unittest.TestCase):
             manager.Server = orig_swift_server
 
     def test_no_daemon(self):
-        class MockServer():
+        class MockServer(object):
 
-            def __init__(self, server):
+            def __init__(self, server, run_dir=manager.RUN_DIR):
                 self.server = server
                 self.called = defaultdict(list)
 
             def launch(self, **kwargs):
                 self.called['launch'].append(kwargs)
+                return {}
 
             def interact(self, **kwargs):
                 self.called['interact'].append(kwargs)
@@ -1441,9 +1619,9 @@ class TestManager(unittest.TestCase):
             manager.Server = orig_swift_server
 
     def test_once(self):
-        class MockServer():
+        class MockServer(object):
 
-            def __init__(self, server):
+            def __init__(self, server, run_dir=manager.RUN_DIR):
                 self.server = server
                 self.called = defaultdict(list)
 
@@ -1455,7 +1633,8 @@ class TestManager(unittest.TestCase):
                     return 0
 
             def launch(self, **kwargs):
-                return self.called['launch'].append(kwargs)
+                self.called['launch'].append(kwargs)
+                return {}
 
         orig_swift_server = manager.Server
         try:
@@ -1478,18 +1657,21 @@ class TestManager(unittest.TestCase):
             manager.Server = orig_swift_server
 
     def test_stop(self):
-        class MockServerFactory():
-            class MockServer():
-                def __init__(self, pids):
+        class MockServerFactory(object):
+            class MockServer(object):
+                def __init__(self, pids, run_dir=manager.RUN_DIR):
                     self.pids = pids
 
                 def stop(self, **kwargs):
                     return self.pids
 
-            def __init__(self, server_pids):
+                def status(self, **kwargs):
+                    return not self.pids
+
+            def __init__(self, server_pids, run_dir=manager.RUN_DIR):
                 self.server_pids = server_pids
 
-            def __call__(self, server):
+            def __call__(self, server, run_dir=manager.RUN_DIR):
                 return MockServerFactory.MockServer(self.server_pids[server])
 
         def mock_watch_server_pids(server_pids, **kwargs):
@@ -1519,6 +1701,14 @@ class TestManager(unittest.TestCase):
             m = manager.Manager(['test'])
             status = m.stop()
             self.assertEquals(status, 1)
+            # test kill not running
+            server_pids = {
+                'test': []
+            }
+            manager.Server = MockServerFactory(server_pids)
+            m = manager.Manager(['test'])
+            status = m.kill()
+            self.assertEquals(status, 0)
             # test won't die
             server_pids = {
                 'test': [None]
@@ -1532,7 +1722,7 @@ class TestManager(unittest.TestCase):
             manager.Server = _orig_server
             manager.watch_server_pids = _orig_watch_server_pids
 
-    # TODO: more tests
+    # TODO(clayg): more tests
     def test_shutdown(self):
         m = manager.Manager(['test'])
         m.stop_was_called = False
@@ -1567,7 +1757,7 @@ class TestManager(unittest.TestCase):
         self.assertEquals(m.start_was_called, True)
 
     def test_reload(self):
-        class MockManager():
+        class MockManager(object):
             called = defaultdict(list)
 
             def __init__(self, servers):
@@ -1602,7 +1792,7 @@ class TestManager(unittest.TestCase):
             # test force graceful
             MockManager.reset_called()
             m = _orig_manager(['*-server'])
-            self.assert_(len(m.servers), 4)
+            self.assertEquals(len(m.servers), 4)
             for server in m.servers:
                 self.assert_(server.server in
                              manager.GRACEFUL_SHUTDOWN_SERVERS)
